@@ -1154,7 +1154,7 @@ jobrep_push_certificates(struct jr_db_handle_s *db_handle, STACK_OF(X509) *px509
     char    *subject_DN = NULL, *issuer_DN = NULL, *not_before_str = NULL, *not_after_str = NULL;
     time_t   not_before = 0, not_after = 0;
     X509    *cert = NULL;
-    unsigned char *serialnr = NULL;
+    char *serialnr = NULL;
 
     if ((db_handle == NULL) || (px509_chain == NULL)) {
         return -1;
@@ -1251,27 +1251,25 @@ fail_jobrep:
     return -1;
 }
 
-unsigned char *
+char *
 jobrep_get_serialnumber_as_string(X509 *cert) {
-    ASN1_INTEGER   *cert_Serial = NULL;
-    unsigned char  *serialNumberDER = NULL, *temp = NULL, *serialStr = NULL;
+    ASN1_INTEGER   *serial = NULL;
     char           *subject_DN = NULL;
-    size_t          len;
-    int             j,serialLen;
+    BIGNUM         *bn_serial;
+    char           *serialStr = NULL;
 
     if (cert == NULL)
         return NULL;
 
-    cert_Serial = X509_get_serialNumber(cert);
-    if (cert_Serial == NULL) {
+    if ( (serial = X509_get_serialNumber(cert)) == NULL) {
         /* For debugging purposes extract the Subject DN */
         subject_DN = X509_NAME_oneline(X509_get_subject_name(cert),NULL,0);
         if (subject_DN) {
-            lcmaps_log(LOG_DEBUG, "%s: certificate passed with subject DN \"%s\" didn't contain a serial number.\n",
+            lcmaps_log(LOG_WARNING, "%s: certificate passed with subject DN \"%s\" didn't contain a serial number.\n",
                        __func__, subject_DN);
             free(subject_DN);
         } else {
-            lcmaps_log(LOG_DEBUG, "%s: certificate passed doesn't have a serialnumber and also lacks a subject DN. " \
+            lcmaps_log(LOG_WARNING, "%s: certificate passed doesn't have a serialnumber and also lacks a subject DN. " \
                                   "This is completely weird and doesn't even look like a certificate. " \
                                   "Are you a waiter because you seem to be feeding me soup?\n",
                                   __func__);
@@ -1279,46 +1277,15 @@ jobrep_get_serialnumber_as_string(X509 *cert) {
         return NULL;
     }
 
-    serialLen = i2c_ASN1_INTEGER(cert_Serial, NULL);
-    if (serialLen <= 0) {
-        lcmaps_log(LOG_INFO, "%s: Conversion of a certificate serial number from ASN1_INTEGER to DER failed\n",
-                             __func__);
-        return NULL;
+    if ( (bn_serial = ASN1_INTEGER_to_BN(serial, NULL)) == NULL ||
+	 (serialStr = BN_bn2hex(bn_serial)) == NULL )
+    {
+	lcmaps_log(LOG_WARNING, "%s: Cannot convert ASN1_INTEGER serial to char *", __func__);
+	return NULL;
     }
-
-    /* Note: serialLen is int and >0 */
-    temp = serialNumberDER = malloc((size_t)serialLen);
-    if (serialNumberDER == NULL) {
-        lcmaps_log(LOG_DEBUG, "%s: Could not allocate %d bytes\n",
-		__func__, serialLen);
-        return NULL;
-    }
-    /* Warning, the temp variable will be displaced, use the serialNumberDER instead. */
-    serialLen = i2c_ASN1_INTEGER(cert_Serial, &temp);
-
-    /* Allocate as a Hex decimal + null-terminator */
-    len = (size_t)serialLen * 2 + 1;
-    serialStr = malloc(len);
-    if (serialStr == NULL) {
-        lcmaps_log(LOG_DEBUG, "%s: Could not allocate %lu bytes\n",
-		__func__, (unsigned long)len);
-        goto cleanup;
-    }
-
-    /* Write the DER encoded serial code in Hex to the serialStr */
-    temp = serialStr;
-    for (j = 0; j < serialLen; j++) {
-        snprintf((char *)temp, len, "%02X", serialNumberDER[j] );
-        temp = &(temp[2]);
-    }
-
-    /* Done */
-    free(serialNumberDER);
+    
+    BN_clear_free(bn_serial);
     return serialStr;
-cleanup:
-    if (serialNumberDER) free(serialNumberDER);
-    if (serialStr) free(serialStr);
-    return NULL;
 }
 
 
